      program fractal_dimension
c     computes C(r) versus r for logistic map
c     compile with "f77 -o frac_dim frac_dim.f -lm"
      implicit none
      integer id_ilogb,i,j,n,s,infty
      parameter(n = 2**10,infty=100)
      integer M(-infty:infty)
      real*8 x(n),r,c(-infty:infty)
      open(10,file='c_r.dat')

c      call logistic(x,n) ! construct n points on strange attractor

c	call random(x,n)

	call Cantor(x,n)

      do s=-infty,infty ! zero accumulators
         M(s) = 0 
         c(s) = 0
      enddo

c     M(s) is number of distinct distances r_ij between 2^(s-1) and 2^s
      do i=1,n 
        do j=1,i-1
          r = abs( x(i) - x(j) )   ! distance between pts on attractor
          s = id_ilogb(r)+1        ! 2^(s-1) < r < and 2^s
          M(s)=M(s)+1         
        enddo
      enddo

c     compute c(r=2^s)
      do i=-infty,infty
         do j=-infty,i
            c(i) = c(i) + M(j)
         enddo
         c(i) = 2.d0*c(i)/dble(n)**2 ! c(infty)=1 for N large
      enddo

c     write out data to c_r.dat for plotting
      do i=-infty,infty
        if (c(i) .ne. 0 .and. c(i) .ne. c(i-1)) then
          write(10,*)2.0**i,real(c(i)) ! write out data in single precision
        endif
      enddo

      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine logistic(x,n)
c     compute points on strange attractor of logistic map
      implicit none
      integer i,n,ntran
      parameter(ntran = 2**10)
      real*8 x(n),mu
      parameter(mu=3.56994567120529687)
      real*8 xmin, xmax
      parameter(xmin=0, xmax=1)  ! write out points over this range
      open(20,file='fractal.dat',status='unknown',form='formatted')     

      x(1)=0.5
      do i=1,ntran ! throw away transient
        x(1) = mu*x(1)*(1.-x(1)) 
      enddo

      do i=1,n-1 ! compute points on attractor
        x(1+i) = mu*x(i)*(1.-x(i))
        if(x(i) .gt. xmin .and. x(i) .lt. xmax) then 
          write(20,*)x(i),x(1+i)
        endif
      enddo
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine random(x,n)
      implicit none
      integer i,n,seed
      real*8 x(n),ran2

      seed=-91501765

      do i=1,n
        x(i) = ran2(seed)
      enddo
      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine cantor(x,n)
c     construct n points lying randomly on the cantor set
c     cantor set consists of a trinary point expansion with 0's and 2's
      implicit none
      integer i,j,n,seed,m
      parameter(m=33) !minimum separation for double precision
      integer a(m)
      real*8 x(n),ran2

      seed=-91501765
      do j=1,n
c       generate m random 0's and 2's to construct cantor number
        do i=1,m
          a(i) = 2*int(2*ran2(seed))
        enddo
        do i=1,m
          x(j)=x(j)+a(i)/3.d0**i
        enddo
      enddo
      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      FUNCTION ran2(idum)
      INTEGER idum,IM1,IM2,IMM1,IA1,IA2,IQ1,IQ2,IR1,IR2,NTAB,NDIV
      REAL*8 ran2,AM,EPS,RNMX
      PARAMETER (IM1=2147483563,IM2=2147483399,AM=1./IM1,IMM1=IM1-1,
     *IA1=40014,IA2=40692,IQ1=53668,IQ2=52774,IR1=12211,IR2=3791,
     *NTAB=32,NDIV=1+IMM1/NTAB,EPS=2.23e-16,RNMX=1.-EPS)
      INTEGER idum2,j,k,iv(NTAB),iy
      SAVE iv,iy,idum2
      DATA idum2/123456789/, iv/NTAB*0/, iy/0/
      if (idum.le.0) then
        idum=max(-idum,1)
        idum2=idum
        do 11 j=NTAB+8,1,-1
          k=idum/IQ1
          idum=IA1*(idum-k*IQ1)-k*IR1
          if (idum.lt.0) idum=idum+IM1
          if (j.le.NTAB) iv(j)=idum
11      continue
        iy=iv(1)
      endif
      k=idum/IQ1
      idum=IA1*(idum-k*IQ1)-k*IR1
      if (idum.lt.0) idum=idum+IM1
      k=idum2/IQ2
      idum2=IA2*(idum2-k*IQ2)-k*IR2
      if (idum2.lt.0) idum2=idum2+IM2
      j=1+iy/NDIV
      iy=iv(j)-idum2
      iv(j)=idum
      if(iy.lt.1)iy=iy+IMM1
      ran2=min(AM*iy,RNMX)
      return
      END
