% Plots log-log plot of C(r) versus r as output by fracdim.f
% Least-squares fit of straight line to middle points of curve
% User needs to specify point range of least-squares fit

load c_r.dat
r=c_r(:,1);
C=c_r(:,2);
n=length(r); % number of (r,C) data points
disp('The number of data points is')
disp(n)

% least squares fit to find power-law exponent
range=input('Enter range of data [rmin,rmax] to attempt least-squares fit: ')
% least squares calculation
ndata=range(2)-range(1)+1; % number of pts to fit 
x = log(r);
y = log(C);
I=zeros(1,4); % zero accumulators
for j=range(1):range(2)
   I(1)=I(1)+x(j)^2;
   I(2)=I(2)+x(j);
   I(3)=I(3)+x(j)*y(j);
   I(4)=I(4)+y(j);
end
delta=ndata*I(1)-I(2)^2;
b=(I(1)*I(4)-I(2)*I(3))/delta;
a=(ndata*I(3)-I(2)*I(4))/delta;
disp('The computed power-law exponent is')
disp(a)

% compute error in power-law exponent
sigma=0;
for j=range(1):range(2)
    sigma = sigma + (y(j)-b-a*x(j))^2;
end
sigma = sqrt( sigma / (ndata-2.) ); % parent standard deviation
sigma = sigma * sqrt(ndata/delta); % uncertainty in slope

disp('with error')
disp(sigma)

% plot data and least-squares fit to a straight line
loglog(r,C,'o')
hold on
yline=exp(b) * r.^a;
loglog(r,yline,'r')

