% Plots log-log plot of C(r) versus r as output by fracdim.f
% Least-squares fit of straight line to middle points of curve
% Finds adjacent points which minimizes error

load c_r.dat
r=c_r(:,1);
C=c_r(:,2);
n=length(r); % number of (r,C) data points
x = log(r);
y = log(C);
disp('The number of data points is')
disp(n)

% least squares fit to find power-law exponent
% loop over m-to-n adjacent points to find minimum error of least-squares fit
m=3;
index=0;
for ndata=m:n % number of adjacent points to fit
for start=1:n+1-ndata % starting position of adjacent points
index=index+1; % loop count
  range(1,index) = start; % first point in range
  range(2,index) = start+ndata-1; % last point in range
% least squares calculation
  I=zeros(1,4); % zero accumulators
  for j=range(1,index):range(2,index)
    I(1)=I(1)+x(j)^2;
    I(2)=I(2)+x(j);
    I(3)=I(3)+x(j)*y(j);
    I(4)=I(4)+y(j);
  end
  delta=ndata*I(1)-I(2)^2;
  b(index)=(I(1)*I(4)-I(2)*I(3))/delta; % y-intercept
  a(index)=(ndata*I(3)-I(2)*I(4))/delta; % power-law exponent

% compute error in power-law exponent
  sigma(index)=0;
  for j=range(1,index):range(2,index)
    sigma(index) = sigma(index) + (y(j)-b(index)-a(index)*x(j))^2;
  end
  sigma(index) = sqrt( sigma(index) / (ndata-2.) );
  sigma(index) = sigma(index) * sqrt(ndata/delta); % uncertainty in slope
   
end
end
% find least-squares fit with slope of smallest uncertainty
[sigma_m,i] = min(sigma);
a_m=a(i);
b_m=b(i);
disp('The computed power-law exponent is')
disp(a_m)
disp('with error')
disp(sigma_m)
disp('range of points which are least-squares fit')
disp(range(1,i))
disp(range(2,i))

% plot data and least-squares fit to a straight line
loglog(r,C,'o')
hold on
yline=exp(b_m) * r.^a_m;
loglog(r,yline,'r')

